package main

import (
	"context"
	"fmt"

	"github.com/project-flogo/rules/common"
	"github.com/project-flogo/rules/common/model"
	"github.com/project-flogo/rules/ruleapi"
)

func main() {

	fmt.Println("** rulesapp: Example usage of the Rules module/API **")

	//Load the tuple descriptor file (relative to GOPATH)
	tupleDescAbsFileNm := "./rulesapp.json"
	tupleDescriptor := common.FileToString(tupleDescAbsFileNm)

	//First register the tuple descriptors
	err := model.RegisterTupleDescriptors(tupleDescriptor)
	if err != nil {
		fmt.Printf("Error [%s]\n", err)
		return
	}

	//Create a RuleSession
	rs, _ := ruleapi.GetOrCreateRuleSession("asession")

	//// rule 1
	r1 := ruleapi.NewRule("R1")
	r1.AddCondition("C1", []string{"t1"}, checkC1, nil)
	r1.SetAction(actionA1)
	r1.SetPriority(1)
	rs.AddRule(r1)

	// rule 2
	r2 := ruleapi.NewRule("R2")
	r2.AddCondition("C2", []string{"t1"}, checkC2, nil)
	r2.SetAction(actionA2)
	r2.SetPriority(2)
	rs.AddRule(r2)

	// rule 3
	r3 := ruleapi.NewRule("R3")
	r3.AddCondition("C3", []string{"t1"}, checkC2, nil)
	r3.SetAction(actionA3)
	r3.SetPriority(3)
	rs.AddRule(r3)

	//Start the rule session
	rs.Start(nil)

	//Now assert a "t1" tuple
	t1, _ := model.NewTupleWithKeyValues("t1", "Tom")
	t1.SetString(nil, "name", "test")
	rs.Assert(nil, t1)

	//unregister the session, i.e; cleanup
	rs.Unregister()
}

func checkC1(ruleName string, condName string, tuples map[model.TupleType]model.Tuple, ctx model.RuleContext) bool {
	fmt.Println("In Condition C1")
	return true
}

func actionA1(ctx context.Context, rs model.RuleSession, ruleName string, tuples map[model.TupleType]model.Tuple, ruleCtx model.RuleContext) {
	fmt.Println("In Action A1 Start")

	// change t1 field
	t1 := tuples["t1"].(model.MutableTuple)
	t1.SetString(ctx, "name", "somethingnew")

	fmt.Println("In Action A1 End")
}

func checkC2(ruleName string, condName string, tuples map[model.TupleType]model.Tuple, ctx model.RuleContext) bool {
	fmt.Println("In Condition C2")
	return true
}

func actionA2(ctx context.Context, rs model.RuleSession, ruleName string, tuples map[model.TupleType]model.Tuple, ruleCtx model.RuleContext) {
	fmt.Println("In Action A2 Start")
	t1 := tuples["t1"]
	val, _ := t1.GetString("name")
	fmt.Println("In Action A2 End ", val)
}

func checkC3(ruleName string, condName string, tuples map[model.TupleType]model.Tuple, ctx model.RuleContext) bool {
	fmt.Println("In Condition C3")
	return true
}

func actionA3(ctx context.Context, rs model.RuleSession, ruleName string, tuples map[model.TupleType]model.Tuple, ruleCtx model.RuleContext) {
	fmt.Println("In Action A3 Start")
	t1 := tuples["t1"]
	val, _ := t1.GetString("name")
	fmt.Println("In Action A3 End ", val)
}
